//header Move.h
//Last changed 04.03.2021

#include "classes2.h"

//-----class Move-------------

//this class describes one Move

// NO modpart and modtype arrays, they are replaced by the Threads::mod_partial, mod_type to be updated only in HistClacChange, to 
//preserve the value for the threads
//in case of no atomic move, when HistCalcChange is not called they are set in RMC_POT

class Move
{
	public:
	Move(SimpleCfg &conf);//Default constructor
	Move(int &ntoocl, int *tclarray, SimpleCfg &conf); //constructor for atomic move
	Move( FNC_POT &fnc, int &ntoocl, int *tclarray, SimpleCfg &conf);//overloaded constructor for custom 'molecular' move
		
	~Move(); //destructor
	static int nmoved;//number of atoms involved in the move
	static int ntypes;//number of atom types (this is needed for the modpart array)
	static int npartials;//number of partials (this is needed for the modpart array)

	static int moveout;//moveout option indicator
	
	static int ndist_bins;//number of histograms bins for the distances generated by the move (same for each histogram)
	
	//swap related
	static int swap;//boolean indicator, whether there can be swaps
	static int tot_moved_atoms;//total number of moved atoms (no swap:nmoved, swap:nmoved+1);
	static int tot_moved_virtuals;//total number of virtual sites, whose position was changed by the move 
	static int nswap_pairs;//number of mixed partials, where swap can occure
	static int *swap_type1;//pointer to the array containing the type of the first atom of the allowed swap pairs
	static int *swap_type2;//pointer to the array containing the type of the second atom of the allowed swap pairs

	static double swap_fraction;//fraction of swaps

	static bool nomove;//no atomic move is performed in this step
	
	//EXAFS E0 shift related
	static bool makeE0shift;//indicator whether to make E0 shift instead of atomic move

	//I(Q) fitting scalable background correction
	static bool makemucorr;//indicator whether to make changes to the mu correction paramter for I(Q) fitting
	static bool makefprimeshift;//indicator whether to make changes to the f' for I(Q) fitting

	//Parameters related to the atoms below cutoff
	static int tooclose_flag;//1 if the too close atoms have been loaded, 0 otherwise
	static int ntooclose;//number of atoms not satisfying the cutoffs
	static int *tclindices;//array of indices of atoms not satisfying the cutoffs
	static int *tclpaircount;//how many too close pair the given atom is involved in

	static double *invmove_binwidth;//array of inverse bin width for the moved distance histograms /type
	static double *moveamp;//array of move amplitudes (in Angstrom)
#ifdef _NO_PERIODIC
	static double R0_red;//reduced R0
#endif
	
	int nparams;//number of parameters of a custom move

	//have been modified by the move
	int ntcl_selected;//number of too close atoms among the moved atoms
	int max_pairs;//the maximum of the too close pairs for an atom
	int *itooclose;//array, indices of the moved atom(s) among the too close atoms, if it was a "too close" one, -1 otherwise
	int *tooclose_index;//array, indices of the moved too close atom(s) in the configuration (starting with 0)
	int *tooclose_remove;//array, indicator, whether the moved too close atom(s) can be removed from the list
	int *ntcl_pair;//array, number of too close pairs the given moved tooclose atom(s) is/are involved in, which can be removed from the list 
	int *tclpair_ind;//indices of too close pairs the given moved atom is involved in, which can be removed from the list 
	int *indices;//array of indices of atoms moved
	int *types;//array of types of the atoms moved
	int *tcl_ind;//array, 1 for each tooclose moved atom, 0 if the moved atom is not too close
#ifdef _NO_PERIODIC
	int *atom_binind_old;//the old index of the bin the moved atoms can be found in
#endif
	double *newpos;//array of new positions of atoms moved
	double *oldpos;//array of old positions of atoms moved


	int update_charge_c;//whether update the charge centre
	int *charge_c_indices;//indices of the charge group centres for the moved atoms (not among the GROMACS charge group but all the chrage groups of the configuration) 
	double *old_charge_c_pos;//old positions of the charge centres
	
	double *unscamp;//array of reduced move amplitudes
	double *custparams;//array of parameters of custom move
	

	//handle to object
	SimpleCfg &config;

	//member functions
	static void SetMoveParams();//Sets the static members
	static int LoadTooClose();//loading the too close atoms from .tca file
	static void ResetTooClose();//resets the parameters connected to the too close atoms to zero
	static void SaveTooClose(const char *file_name="");//saving the too close atoms to .tca file
	void RemoveTooClosePairs(int iselected);//removing the other atom of the too close pair from the lists, if the distance ids now above cutoff
	void Copy(Move &dest);
	//reset the new positions in case the move is rejected
	void ResetCoord(FNC_POT *fnc=0);


#ifdef _NO_PERIODIC
	void ResetBinInd();//restoring the original central nin indices, if the move was not accepted
#endif
	
	//********function MakeMove******************
	void MakeMove(SimpleCfg &config, longint &seed, FNC_POT *fnc=0);//normal atomic
	void MakeMove(SimpleCfg &config, FNC_POT &fnc, longint &seed);//molecular
	//*******************************************	

};

//destructor of the Move class
inline Move::~Move()
{
	delete [] itooclose;
	delete [] tooclose_index;
	delete [] tooclose_remove;
	delete [] ntcl_pair;
	delete [] tclpair_ind;
	delete [] tcl_ind;//array, 1 for each tooclose moved atom, 0 if the moved atom is not too close
	delete [] unscamp;
	delete [] indices;
	delete [] types;
	delete [] newpos;
	delete [] oldpos;
	delete [] custparams;
#ifdef _NO_PERIODIC
	delete [] atom_binind_old;
#endif
	if (old_charge_c_pos!=NULL)
		delete [] old_charge_c_pos;
	old_charge_c_pos=0;
	if (charge_c_indices!=NULL)
		delete [] charge_c_indices;
	charge_c_indices=0;
	//This is needed to ensure the proper behaviour of the Copy(Move &dest) function
	tcl_ind=0;
	itooclose=0;
	tooclose_index=0;
	tooclose_remove=0;
	ntcl_pair=0;
	tclpair_ind=0;
	indices=0;
	types=0;
	newpos=0;
	oldpos=0;
	unscamp=0;
	custparams=0;
#ifdef _NO_PERIODIC
	atom_binind_old=0;
#endif
	
	if(::debug) cout<<"Move destructor"<<endl;
};


