//source: global.cpp
//Last changed 12.12.2022

#define _DEF_FILES //not redefine the file names included through global.h
#include "altern.h"
#include <map>
#include <set>
#include <string>

//these are used at may places in the createfree_... routines, easier to have it here once, and only use the apprpriate element, where needed
std::string sigma_text[3] ={
	"standard deviation (this or any alternative is) **MANDATORY ITEM**",
	"standard deviation and leading series for scalable sigma (this or any alternative is) **MANDATORY ITEM**",
	"scalable sigma, sets starting chi2 of this set or constraint to the proportion given by this scalable sigma of the leading series' starting chi2, (this or any alternative is) **MANDATORY ITEM**" };

std::string alternative_sigma_text[3] = {
	"\t\t\t! alternatives: (..._)SIGMA-MASTER (standard deviation and leading series for scalable sigma) or\n\t\t\t! (..._)SIGMA-SCALABLE (sets starting chi2 of this set or constraint to the proportion given by this scalable sigma\n\t\t\t! of the leading series' starting chi2",
	"\t\t\t! alternatives: (..._)SIGMA (standard deviation) or\n\t\t\t! (..._)SIGMA-SCALABLE (sets starting chi2 of this set or constraint to the proportion given by this scalable sigma of the leading series' starting chi2",
	"\t\t\t! alternatives: (..._)SIGMA (standard deviation) or\n\t\t\t! (..._)SIGMA-MASTER (standard deviation and leading series for scalable sigma"};

std::string potential_types[11] = { "","LJ","","","","","","","","","TABULATED" };
//needed for the tags and keyword tables
//they are placed here,as they could not be referenced, if they were defined in global.h
//implemented as maps, to make it easy to search for tags and keywords


const std::map<std::string, tag_table_type> tag_table({
	{"GENERAL",{"1","General parameters of the run"}},
	{"EXP",{"Number of experiments","Parameters for one experimental set"}},
	{"COS",{"Number of cosine distribution of bond angles constraints","Cosine distribution of bond angles constraint"}},
	{"COORD",{"Number of coordination constraints","Coordination constraint"}},
	{"AVCOORD",{"Number of average coordination constraints","Average coordination constraint"}},
	{"CONC",{"Number of common neighbour constraints","Common neighbour constraint"}},
	{"SNC",{"Number of second neighbour constraints","Second neighbour constraint"}},
	{"BVS",{"Number of bond valence sum constraints","Bond valence sum constraint"}},
	{"SWAP",{"1","To use swapping of atoms"}},
	{"CUSTMOVE",{"1","To use custom move"}},
	{"NBPOT",{"1","To use non-bonded potential"}},
	{"BPOT",{"1","To use bonded potential"}},
	{"AENET",{"1","To use AENET"}},
	{"LOCINV",{"1","To use local invariance calculation"}},
	{"NOPER",{"1","To use no periodic boundary conditions"}},
	{"VIBRAMP",{"1","To use bonded potential"}},
	{"END",{"1","End of file processing"}}
	});



const std::map<std::string, key_table_type> key_table({
	{"CUT-OFF",{{},"OPTIONAL",{"GENERAL"},{"npartials real numbers, MIN or MIN-ALL "},"MIN-ALL","Distance in A between the atom pairs for each partial, if MIN or MIN-ALL is given,\n\tthen the minimal distance for each partial is determined automatically and set,\n\tsee the difference in the manual"}},
	{"TITLE",{{},"OPTIONAL",{"GENERAL"},{"arbitrary string"},"","Title of the run"}},
	{"NDENS",{{"HALFBOX"},"OPTIONAL",{"GENERAL"},{"real number"},"calculated from *.cfg","Number density in 1/A^3"}},
	{"HALFBOX",{{"NDENS"},"OPTIONAL",{"GENERAL"},{"real number"},"read from *.cfg","Half length of the simulation box in A"}},
	{"MAX-MOVES",{{},"OPTIONAL",{"GENERAL"},{"ntypes real number"},std::to_string(MAX_MOVES_DEF),"Maximum move for each atom type in A"} },
	{"R-SPACING",{{},"OPTIONAL",{"GENERAL","EXP"},{"real number"},std::to_string(RSPACING_DEF),"Size of the histogram bins in A"}},
	{"MOVEOUT",{{},"OPTIONAL",{"GENERAL"},{"0","1"},std::to_string(MOVEOUT_DEF),"Whether to use moveout option (0: no, 1: yes)"} },
	{"TOOCLOSE-FRACTION",{{},"OPTIONAL",{"GENERAL"},{"real number"},std::to_string(TOOCLOSE_FRACTION_DEF),"What fraction of the moves should be from too close atoms in case of moveout option (between 0-1)"} },
	{"PRINT-STEP",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(PRINTSTEP_DEF),"Number of generated moves between printing to general output"}},
	{"RUN-TIME",{{"LAST-GEN","LAST-ACC"},"OPTIONAL",{"GENERAL"},{"real number"},std::to_string(TIMELIM_DEF),"Run time in minutes, ignored in _TEST_MODE"}},
	{"LAST-GEN",{{"RUN-TIME","LAST-ACC"},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(LAST_MOVE_DEF),"Number of generated moves for the run,\n\t(in _TEST_MODE only LAST-ACC is an alternative)"}},
	{"LAST-ACC",{{"RUN-TIME","LAST-GEN"},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(LAST_MOVE_DEF),"Number of accepted moves for the run,\n\t(in _TEST_MODE only LAST-GEN is an alternative)"}},
	{"SAVE-TIME",{{},"OPTIONAL",{"GENERAL"},{"real number"},std::to_string(TIMESAVE_DEF),"Saving to the disk in minutes"} },
	{"HST_BUFFSIZE",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(HIST_BUFF_DEF),"Size of the history buffer, deprecated"}},
	{"HST-BUFFSIZE",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(HIST_BUFF_DEF),"Size of the history buffer"}},
	{"HST-STEP-FACTOR",{{"HST-SAVE-TIME"},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(HIST_STEP_DEF),"Number of savings to disc between history buffering"} },
	{"HST-SAVE-TIME",{{"HST-STEP-FACTOR"},"OPTIONAL",{"GENERAL"},{},"HST-STEP-FACTOR or its default: " + std::to_string(HIST_STEP_DEF),"History is buffered with this interval, should be or set to be the integer multiple of SAVE-TIME"}},
	{"COLL-NUMBER",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(CFG_COLL_DEF),"Number of configurations to collect at the end of the run"} },
	{"COLL-STEP-FACTOR",{{"COLL-SAVE-TIME"},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(CFG_COLL_FREQ_DEF),"A configuration will be saved at each COLL-STEP-FACTOR*SAVE-TIME"} },
	{"COLL-SAVE-TIME",{{"COLL-STEP-FACTOR"},"OPTIONAL",{"GENERAL"},{"integer number"},"COLL-STEP-FACTOR or its default: " + std::to_string(CFG_COLL_FREQ_DEF),"A configuration will be saved at each COLL-SAVE-TIME, \n\tit should be set to an integer multiple of SAVE-TIME"}},
	{"FNC-TYPE",{{},"OPTIONAL",{"GENERAL"},{"NONE","NORMAL","ADJUST","MOVE-IN"},"NONE","Type of the FNC constraint"} },
	{"BIN-SHIFT",{{},"OPTIONAL",{"GENERAL"},{"real number"},std::to_string(BINSHIFT_DEF),"Number of bins to leave out from the calculation at the beginning of the histogram"} },
	{"XMAX-FACTOR",{{},"OPTIONAL",{"GENERAL"},{"real number"},std::to_string(XMAX_DEF),"Largest distance between the particles to include in the histogram \n\tcalculation in reduced unit, maximum sqrt(3) "}},
	{"RELOAD",{{},"OPTIONAL",{"GENERAL"},{"0","1"},std::to_string(RELOAD_DEF),"Whether to load the histogram from disc (0: no, 1: yes)"} },
	{"ATOMS-IN-GRIDCELL",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(MAX_GRIDATOM_DEF),"Number of atoms in a grid cell"}},
	{"THREADS",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(NTHREADS_DEF),"Number of threads to use"} },
	{"CUSTOM-SFACTORTABLE",{{},"OPTIONAL",{"GENERAL"},{"file name"},"built-in is used","Use the given file for custom surface factors"} },
	{"CHEMICAL-SYMBOLS",{{},"OPTIONAL",{"GENERAL"},{"ntypes string, case sensitive"},"","Chemical symbols of the atom types in the order they are represented\n\tin the configuration, needed if X-ray atomic scattering factors, Compton-scattering or neutron coefficients are calculated"} },
	{"CREATE-OUT",{{},"OPTIONAL",{"GENERAL"},{"0","1"},std::to_string(OLD_OUT_DEF),"Whether to write old format *.out file (0: no, 1: yes)"} },
	{"PPCF-AVERAGE",{{},"OPTIONAL",{"GENERAL"},{"0","1"},std::to_string(SUM_PPCF_DEF),"Whether to write summed ppcfs to the *.ppcf file as well at each saving (0: no, 1: yes)"}},
	{"EXAFS-SHIFTSTEP",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(E0_SHIFTSTEP_DEF),"Shift E0 and so therefore k for selected EXAFS data set in each EXAFS-SHIFTSTEP step"}},
	{"AXS-SHIFTSTEP",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(AXS_SHIFTSTEP_DEF),"Shift f' for AXS I(Q) fitting sets in each AXS-SHIFTSTEP step"}},
	{"MAX-NITER-NONLIN",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(NONLIN_MAX_NIT_DEF),"Maximum number of iteration in case of I(Q) fitting with non-linear regression"}},
	{"EPSILON-NONLIN",{{},"OPTIONAL",{"GENERAL"},{"real number"},std::to_string(NONLIN_EPSILON_DEF),"Iteration stops, if the fitted parameters change less than this"}},
	{"LAMBDA-NONLIN",{{},"OPTIONAL",{"GENERAL"},{"real number"},std::to_string(NONLIN_LAMBDA_DEF),"Fudge factor for Levenberg-Marquardt non-linear regression"}},
	{"FACTOR-NONLIN",{{},"OPTIONAL",{"GENERAL"},{"real number"},std::to_string(NONLIN_FACTOR_DEF),"Factor to change fudge for Levenberg-Marquardt non-linear regression"}},
	{"TERMINATE-NONLIN",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(NONLIN_TERMINATE_DEF),"Terminate after this many successive moves ended with failed non-linear regression"}},
	{"I(Q)BACKG-STEP",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(IQ_BACKG_CORR_STEP_DEF),"Change the background correction factor mu in every I(Q)BACKG-STEP, step"}},
	{"WRITE-LOG",{{},"OPTIONAL",{"GENERAL"},{"0","1"},std::to_string(WRITE_LOG_DEF),"Whether to create a log file, only for developers (0: no, 1: yes)"}},
	{"LOG-NONLIN-STEPS",{{},"OPTIONAL",{"GENERAL"},{"0","1"},std::to_string(LOG_NONLIN_STEPS_DEF),"Write non-linear regression steps, if there is any (0: no, 1: yes)"}},
	{"DEBUG",{{},"OPTIONAL",{"GENERAL"},{"0","1"},std::to_string(DEBUG_DEF),"Write debug information (0: no, 1: yes)"}},
	{"WRITE-EXAFS-COEFFS",{{},"OPTIONAL",{"GENERAL"},{"0","1"},std::to_string(WRITE_EXAFS_COEFFS_DEF),"Whether to write the EXAFS coefficients to the *.expt file, only active if E0 shift is applied\n\t(0: no, 1: yes)"}},
	{"R-SWITCH-POWER",{{},"OPTIONAL",{"GENERAL"},{"integer number"},std::to_string(R_SWITCH_POWER_DEF),"Modify the default value for the _R_SWITCH_GR_MIN_1 compiler option, if it is on, then\n\texperimental g(r) data sets will be interpreted as (r^r_switch_power)*[g(r)-1],\n\tand this will be calculated and fitted"}},
	{"TYPE",{{},"MANDATORY",{"EXP"},{"GR","ND","XRD","EDIFF","EXAFS"},"","Type of the experimental set"}},
	{"DATAFILE",{{},"MANDATORY",{"EXP"},{"arbitrary string"},"","Name of the data file"}},
	{"SIGMA",{{"SIGMA-SCALABLE","SIGMA-MASTER"},"One of it is mandatory",{"EXP","COS","AENET"},{"real number"},"","Sigma parameter"}},
	{"SIGMA-MASTER",{{"SIGMA","SIGMA-SCALABLE"},"One of it is mandatory",{"EXP","COS","AENET"},{"real number"},"","Sigma parameter for the leading series"}},
	{"SIGMA-SCALABLE",{{"SIGMA","SIGMA-MASTER"},"One of it is mandatory",{"EXP","COS","AENET"},{"real number"},"","Scalable sigma parameter"}},
	{"ISOTOPE-COUNT_SYMBOLS_RATIOS",{{},"OPTIONAL for TYPE=ND",{"EXP"},{"integer, string(s), real number(s)"},"natural isotope abundance","For a chemical element use as many isotopes for a neutron data set as given by isotope count,\n\tthe isotopes should be specified in the format like C14, sum of the ratios has to be 1!\n\tCan give as many lines as the number of components for a neutron data set. "}},
	{"READ-COEFFS",{{},"OPTIONAL for TYPE=XRD",{"EXP"},{"0","1"},std::to_string(READ_COEFFS_DEF)+" from free format *.dat version #003","Indicator whether to read the X-ray coefficients from the experimental data file\n\tin case of F(Q)-fitting or f(Q)-s in case of I(Q) fitting for the given X-ray data set (0: no, 1: yes)\n\tFor free format *.dat version #002 reading will be the default to maintain downward compatibility"}},
	{"I(Q)_A_B_ALPHA",{{},"OPTIONAL for TYPE=XRD",{"EXP"},{"one integer and optionally three real numbers"},std::to_string(IQFIT_DEF)+", -, -, " + std::to_string(IQ_ALPHA_DEF),"Indicator to fit I(Q) instead of F(Q), if custom values are needed should be supplied here.\n\tIf b and alpha custom value is needed, something for a value should be given as well!"}},
	{"I(Q)BACKG_MU_DMU-MAX",{{},"OPTIONAL for TYPE=XRD",{"EXP"},{"one integer and optionally two real numbers"},std::to_string(IQBACKGCORR_DEF)+", "+ std::to_string(IQ_MU_DEF) + ", " + std::to_string(IQ_DMU_MAX_DEF),"Indicator to use background correction for I(Q) fitting, if custom values used,\n\tshould be supplied here: mu and dmu_max."}},
	{"FPRIME-INDEX_FPRIME",{{},"OPTIONAL for TYPE=XRD",{"EXP"},{"one integer and one real number"},"","In case of X-ray data sets, index of the atom type to use f' for (starting with 1),\n\tthen f' for this atom type in this experiment. If index 0 no f' is used.\n\tf' is the real part of the E-dependent scattering factor part for this element,\n\tf(Q)+f'(E) is used instead of f(Q) for coefficient calculation for this type"}},
	{"AXS-INDEX_FPRIME_FRACTION",{{},"OPTIONAL for TYPE=XRD",{"EXP"},{"one integer, two real numbers"},"","Only for I(Q) fitting, indicating this is an anomalous X-ray scattering data set.\n\tIndex is the atom type index (starting with 1) for the edge,\n\tFPRIME is the real part of the E-dependent scattering factor\n\tf'(E) for this element, f(Q)+f'(E) is used during coefficient calculation. FRACTION indicates the\n\tfraction of f'(E): varying f'(E) between f'(E)-f'(E)*FRACTION -> f'(E)+f'(E)*FRACTION\n\tin every AXS-SHIFTSTEP. step."}},
	{"COMPTON",{{},"OPTIONAL for TYPE=XRD",{"EXP"},{"0","1"},std::to_string(COMPTON_DEF),"Only for I(Q) fitting, indicating whether Compton-scattering contribution should be used (0: no, 1: yes)"}},
	{"ISOTOPE-COUNT_SYMBOLS_RATIOS",{{},"OPTIONAL for TYPE=ND",{"EXP"},{"integer, string, real number"},"natural abundance","In case of neutron data, if the coeffs should be calculated by the program, and not natural abundance\n\tshould be used for a component. As many lines, as components with not natural abundance."}},
	{"PARTIAL-COEFFS",{{},"MANDATORY for TYPE=GR and optional for ND",{"EXP"},{"real number"},"","Coefficients for the partials"}},
	{"ABSORBER-TYPE",{{},"MANDATORY for TYPE=EXAFS",{"EXP"},{"integer number"},"","Type of the absorbing particle for EXAFS"}},
	{"DELTAE0_NGRID",{{},"OPTIONAL for TYPE=EXAFS",{"EXP"},{"real, integer number"},std::to_string(MAX_E0_SHIFT_DEF)+", "+std::to_string(EXAFS_NGRID_DEF)+" no shift","Maximum E0-shift in eV and number of grid points for the shift in one direction in case of EXAFS"}},
	{"E0GRID-FROM_TO_START",{{},"OPTIONAL for TYPE = EXAFS",{"EXP"},{"three integer numbers"},std::to_string(EXAFS_NGRID_DEF)+", "+std::to_string(EXAFS_NGRID_DEF) + ", " + std::to_string(EXAFS_NGRID_DEF) +": no shift, or symmetric range, if DELTAE0_NGRID is given","Using only the range of E0 grid points for EXAFS specified by FROM, TO,\n\tshould be between -NGRID -> +NGRID, therefore DELTAE0_NGRID is needed as well.\n\tOptionally, START is the grid index of the starting grid point."}},
	{"BACKSCATT-FILE",{{},"MANDATORY  for TYPE=EXAFS",{"EXP"},{},"","Name of the file containing the (k,r)-dependent coefficients"}},
	{"CONST-SUBTRACT",{{},"OPTIONAL",{"EXP"},{"real number"},std::to_string(SUBTRACT_DEF),"Constant to subtract from the data points"}},
	{"RENORM",{{},"OPTIONAL",{"EXP"},{"0","1"},std::to_string(RENORM_DEF),"Whether to use renormalization coefficient (0: no, 1: yes)"}},
	{"POLY-BACK-FLAGS",{{},"OPTIONAL",{"EXP"},{"0","1"},std::to_string(RENORM_DEF),"Whether to use polinomial background correction,\n\ta value for each (offset, linear, quadratic, cubic) (0: no, 1: yes)"}},
	{"COMPTON-SUPR-COEFF",{{},"OPTIONAL for TYPE=XRD",{"EXP"},{"0","1"},std::to_string(RENORM_DEF),"Whether to vary alpha during I(Q) fit\n\t(0: no, 1: yes)"}},
	{"USE-RFACTOR",{{},"OPTIONAL",{"EXP"},{"0","1"},std::to_string(USE_RFACTOR_DEF),"Whether to use R-factor instead of normal chi2 for this set (0: no, 1: yes)"}},
	{"POINT-RANGE",{{"R-RANGE","Q-RANGE","G-RANGE","K-RANGE"},"OPTIONAL",{"EXP"},{"2 integer numbers"},"whole data range","First and last data points to use"}},
	{"R-RANGE",{{"POINT-RANGE","R-POINT-RANGE"},"OPTIONAL",{"EXP"},{"2 real numbers"},"whole data range","The value of the first and last r-data points to use for a g(r) data set"}},
	{"Q-RANGE",{{"POINT-RANGE"},"OPTIONAL",{"EXP"},{"2 real numbers"},"whole data range","The value of the first and last Q-data points to use for a neutron or x-ray data set"}},
	{"G-RANGE",{{"POINT-RANGE"},"OPTIONAL",{"EXP"},{"2 real numbers"},"whole data range","The value of the first and last g-data points to use for an electron diffraction data set"}},
	{"K-RANGE",{{"POINT-RANGE"},"OPTIONAL",{"EXP"},{"2 real numbers"},"whole data range","The value of the first and last k-data points to use for an EXAFS data set"}},
	{"CHIK-POWER",{{},"OPTIONAL",{"EXP"},{"integer number"},std::to_string(EXAFS_CHI2_POWER_DEF),"(k/k_MAX)^CHIK-POWER for weighting the EXAFS data"}},
	{"R-POINT-RANGE",{{"R-RANGE"},"OPTIONAL",{"EXP"},{"2 real numbers"},"whole data range","First and last r-data points to use from the EXAFS (k,r) coefficient file"}},
	{"WRITE-CNC-DETAIL",{{},"OPTIONAL",{"COORD"},{"0","1"},std::to_string(WRITE_CNC_DETAIL_DEF),"Whether to write the details about the neighbours into the *.cncd file"}},
	{"CENT-TYPE",{{},"MANDATORY",{"COS","COORD","AVCOORD"},{"integer number"},"","Type of central atom"}},
	{"NEIGH-TYPE_FROM_TO",{{},"MANDATORY",{"COS","COORD","AVCOORD"},{"an integer and two real numbers"},"","Type of neighbour atom, minimum and maximum distances in A\n\tInclude twice for COS, once for AVCOORD, for COORD as many times as the number of neighbour types"}},
	{"DISTRIB-TYPE",{{""},"OPTIONAL",{"COS"},{"UNIFORM","GAUSSIAN","ABSENT","filename"},DISTRIB_TYPE_DEF,"Distribution type for cosine distribution constraint"}},
	{"DISTRIB-DEGREES",{{""},"MANDATORY if DISTRIB-TYPE is not read from file",{"COS"},{"real number"},"","Desired angle in degree"}},
	{"DISTRIB-WIDTH",{{""},"MANDATORY if DISTRIB-TYPE is not read from file",{"COS"},{"real number"},"","Wcontrol parameter"}},
	{"DCOSTH",{{""},"OPTIONAL",{"COS"},{"real number"},std::to_string(DCOSTHETA_DEF),"Spacing in cos(theta) space, has to be the same for all the COS constraints \n\texcept DISTRIB-TYPE = filename. If different, last value is used"}},
	{"COORDNUM_FRACT_SIGMA",{{"COORDNUM_FRACT_SIGMA-SCALABLE","COORDNUM_FRACT_SIGMA-MASTER"},"One of them is mandatory",{"COORD","CONC","SNC"},{"integer, real, real number"},"","Desired coordination number, fraction and sigma parameter"}},
	{"COORDNUM_FRACT_SIGMA-MASTER",{{"COORDNUM_FRACT_SIGMA","COORDNUM_FRACT_SIGMA-SCALABLE"},"One of them is mandatory",{"COORD","CONC","SNC"},{"integer, real, real number"},"","Desired coordination number, fraction and sigma parameter for leading series"}},
	{"COORDNUM_FRACT_SIGMA-SCALABLE",{{"COORDNUM_FRACT_SIGMA","COORDNUM_FRACT_SIGMA-MASTER"},"One of them is mandatory",{"COORD","CONC","SNC"},{"integer, real, real number"},"","Desired coordination number, fraction and scalable sigma parameter"}},
	{"COORDNUM_SIGMA",{{"COORDNUM_SIGMA-SCALABLE","COORDNUM_SIGMA-MASTER"},"One of them is mandatory",{"AVCOORD"},{"integer, real, real number"},"","Desired average coordination number and sigma parameter"}},
	{"COORDNUM_SIGMA-MASTER",{{"COORDNUMBSIGMA","COORDNUM_SIGMA-SCALABLE"},"One of them is mandatory",{"AVCOORD"},{"integer, real, real number"},"","Desired average coordination number and sigma parameter for leading series"}},
	{"COORDNUM_SIGMA-SCALABLE",{{"COORDNUM_SIGMA","COORDNUM_SIGMA-MASTER"},"One of them is mandatory",{"AVCOORD"},{"integer, real, real number"},"","Desired average coordination number and scalable sigma parameter"}},
	{"PTYPE1_PTYPE2_FROM_TO",{{},"MANDATORY",{"CONC"},{"two integer and two real numbers"},"","Type of primary1 and primary2 atoms, minimum and maximum distance in A from each other"}},
	{"STYPE_FROM1_TO1_FROM2_TO2",{{},"MANDATORY",{"CONC"},{"an integer and two mandatory and two optional real numbers"},"","Type of secondary atom, minimum and maximum distance in A to P1, to P2,\n\t(minimum and maximum distance to P2, if not given same used as for P1)\n\tInclude as many times, as the number of secondary types."}},	
	{"CTYPE_FTYPE_FROM_TO",{{},"MANDATORY",{"SNC"},{"two integer and two real numbers"},"","Type of central and first neighbour atoms, minimum and maximum distance in A from each other"}},
	{"STYPE_FROM_TO",{{},"MANDATORY",{"SNC"},{"an integer and two real numbers"},"","Type of second neighbour atom, minimum and maximum distance in A to first neighbour,\n\tInclude as many times, as the number of different second neighbour types."}},
	{"CENT-TYPE_CHARGE",{{},"MANDATORY",{"BVS"},{"two integer numbers, second optional"},"","Type of central atom, charge of central atom (only needed, if default value for R0 and b should be used"}},
	{"NEIGH-TYPE_RMAX_R0_B_CHARGE",{{},"MANDATORY",{"BVS"},{"an integer, two real, one optional real and one optional integer number"},"","Type of neighbour atom, maximum distance in A, R0 (bond length) in A (optional to give value to use),\n\tb parameter in A (optional), allocated charge (optional). Give -1 for R0, and/or b if default should be used,\n\tand give the assumed charge of the neighbour for R0 default value\n\tdetermination. Include as many times, as the number of different neighbour types."}},
	{"VALENCE_SIGMA",{{"VALENCE_SIGMA-SCALABLE","VALENCE_SIGMA-MASTER"},"One of them is mandatory",{"BVS"},{"two real numbers"},"","Desired valence, sigma parameter."}},
	{"VALENCE_SIGMA-MASTER",{{"VALENCE_SIGMA","VALENCE_SIGMA-SCALABLE"},"One of them is mandatory",{"BVS"},{"two real numbers"},"","Desired valence, sigma parameter for leading series."}},
	{"VALENCE_SIGMA-SCALABLE",{{"VALENCE_SIGMA","VALENCE_SIGMA-MASTER"},"One of them is mandatory",{"BVS"},{"two real numbers"},"","Desired valence, scalable sigma parameter."}},
	{"FRACT",{{""},"MANDATORY",{"SWAP"},{"real number"},"","Swap fraction"}},
	{"PAIRS",{{""},"MANDATORY",{"SWAP"},{"ALL","Type-ANY","Type1-Type2"},"","Atom types to swap"}},
	{"NMOVED-ATOMS",{{""},"MANDATORY",{"CUSTMOVE"},{"integer number"},"","Number of atoms to move in a single move"}},
	{"LOC-MODE",{{""},"OPTIONAL",{"LOCINV"},{"BIN-BASED","DISTANCE-BASED"},"DISTANCE-BASED","Calculation mode for local invariance: bin-based (0), distance-based (1)"}},
	{"LOC-INT-FROM_SIGMA",{{"LOC-INT-FROM_SIGMA-MASTER","LOC-INT-FROM_SIGMA-SCALABLE"},"One of them is mandatory",{"LOCINV"},{"two real numbers"},"","For BIN-BASED calculation give it once: starting reduced distance for the interval,\n\tfor DISTANCE-BASED calculation as many times as the number of intervals:\n\tstarting atom ratio for the interval; sigma"}},
	{"LOC-INT-FROM_SIGMA-MASTER",{{"LOC-INT-FROM_SIGMA","LOC-INT-FROM_SIGMA-SCALABLE"},"One of them is mandatory",{"LOCINV"},{"two real numbers"},"","For BIN-BASED calculation give it once: starting reduced distance for the interval,\n\tfor DISTANCE-BASED calculation as many times as the number of intervals:\n\tstarting atom ratio for the interval; leading series sigma"}},
	{"LOC-INT-FROM_SIGMA-SCALABLE",{{"LOC-INT-FROM_SIGMA-MASTER","LOC-INT-FROM_SIGMA"},"One of them is mandatory",{"LOCINV"},{"two real numbers"},"","For BIN-BASED calculation give it once: starting reduced distance for the interval,\n\tfor DISTANCE-BASED calculation as many times as the number of intervals:\n\tstarting atom ratio for the interval; scalable sigma"}},
	{"LOC-INT-TO",{{""},"OPTIONAL",{"LOCINV"},{"real number"},"Same as for normal histogram ","Include only once setting the end of the (last) interval.\n\tLargest reduced distance included in local invariance histogram calculation"}},
	{"LOC-R-SPACING",{{""},"OPTIONAL",{"LOCINV"},{"real number"},"default normal histogram bin size","Local invariance histogram spacing in A"}},
	{"CUSTOM-LINE",{{""},"MANDATORY for new format custom files for each line",{"CUSTOM"},{""},"","For new format custom move file, CUSTOM-LINE = should proceed the data for each line"}},
	{"GAUSSIAN-SIGMA",{{""},"MANDATORY",{"VIBRAMP"},{"ntypes real numbers"},"","Sigma parameter in A controlling the width of the Gaussian distribution, \n\tone values for each atom type"}},
	{"SAMPLE-RADIUS",{{""},"MANDATORY",{"NOPER"},{"real number"},"","Radius of the spherical sample in A"}},
	{"RECENTRE-SPHERE",{{""},"OPTIONAL",{"NOPER"},{"0","1"},"0","Whether to recentre the spherical sample in the box (0: no, 1: yes)"}},
	{"NB-TYPE",{{""},"OPTIONAL",{"NBPOT"},{"LJ","TABULATED"},potential_types[POT_TYPE_DEF],"Type of the non-bonded potential"}},
	{"NGR-TYPES",{{""},"OPTIONAL",{"NBPOT"},{"integer number"},"number of RMC types","Number of GROMACS types"}},
	{"LJ-POW",{{""},"OPTIONAL",{"NBPOT"},{"integer number"},std::to_string(LJ_REP_N_DEF),"Power of the LJ repulsion term"}},
	{"1-4-SCALE-VDW_COUL",{{""},"OPTIONAL",{"NBPOT"},{"two real numbers"},std::to_string(FUDGE14_DEF),"Scaling factor for the 1-4 vdW and Coulomb interactions"}},
	{"CHI-LOWLIMIT-RATIO",{{""},"OPTIONAL",{"NBPOT"},{"real number","NONE"},"NONE","The chi2_pot will not decrease under fraction*starting chi2_pot"}},
	{"VDW-SIGMA",{{"SIGMA-OVERALLPOT"},"Either this and COUL-SIGMA, or SIGMA-OVERALLPOT mandatory",{"NBPOT"},{"One or npartials real numbers.\n\tPrecede the number with m for leading series, with s for scalable"},"","Weight parameters for the vdW interactions;\n\tif  NB weight mode=1: npartials number is expected, otherwise one value"}},
	{"COUL-SIGMA",{{"SIGMA-OVERALLPOT"},"Either this and VDW-SIGMA, or SIGMA-OVERALLPOT mandatory",{"NBPOT"},{"One or npartials real numbers.\n\tPrecede the number with m for leading series, with s for scalable"},"","Weight parameters for the Coulomb interactions;\n\tif  NB weight mode=1: npartials number is expected, otherwise one value.\n\tIf key not given, vdW-SIGMA weight will be applied!"}},
	{"SIGMA-OVERALLPOT",{{"VDW-SIGMA and COUL-SIGMA"},"Either this, or VDW-SIGMA and COUL-SIGMA mandatory",{"NBPOT"},{"real number"},"","Weight parameters for all the potential-related interactions (NB weight mode:2)"}},
	{"LJ-C6-EACH",{{"LJ-C6-GMEAN","LJ-SIG-AMEAN","LJ-SIG-GMfEAN"},"One of them is mandatory",{"NBPOT"},{"nGROMACS partials real numbers"},"","LJ first potential parameters in kJ*A^6/mol, used for combination rule=0, see the manual"}},
	{"LJ-C6-GMEAN",{{"LJ-C6-EACH","LJ-SIG-AMEAN","LJ-SIG-GMEAN"},"One of them is mandatory",{"NBPOT"},{"nGROMACS types real numbers"},"","LJ first potential parameters in kJ*A^6/mol, used for combination rule=1, see the manual"}},
	{"LJ-SIG-AMEAN",{{"LJ-C6-EACH","LJ-C6-GMEAN","LJ-SIG-GMEAN"},"One of them is mandatory",{"NBPOT"},{"nGROMACS types real numbers"},"","LJ sigma parameters in A, used for combination rule=2, see the manual"}},
	{"LJ-SIG-GMEAN",{{"LJ-C6-EACH","LJ-C6-GMEAN","LJ-SIG-AMEAN"},"One of them is mandatory",{"NBPOT"},{"nGROMACS types real numbers"},"","LJ sigma parameters in A, used for combination rule=3, see the manual"}},
	{"LJ-CPOW-EACH",{{"LJ-CPOW-GMEAN","LJ-EPS-GMEAN"},"One of them is mandatory",{"NBPOT"},{"nGROMACS partials real numbers"},"","LJ second potential parameter in kJ*A^12/mol, used for combination rule=0, see the manual"}},
	{"LJ-CPOW-GMEAN",{{"LJ-CPOW-EACH","LJ-EPS-GMEAN"},"One of them is mandatory",{"NBPOT"},{"nGROMACS types real numbers"},"","LJ second parameters in kJ*A^6/mol, used for combination rule=1, see the manual"}},
	{"LJ-EPS-GMEAN",{{"LJ-CPOW-EACH","LJ-CPOW-GMEAN"},"One of them is mandatory",{"NBPOT"},{"nGROMACS types real numbers"},"","LJ epsilon parameters in A, used for combination rule=2 and 3, see the manual"}},
	{"CUTOFF-VDW_COUL",{{""},"OPTIONAL",{"NBPOT"},{"two real numbers"},"1.0 in reduced unit for both corresponding to boxedge A","Cutoff for the vdW and Coulomb interactions in A"}},
	{"CUTOFF-TAB",{{""},"OPTIONAL",{"NBPOT"},{"real number"},"1.0 in reduced unit corresponding to boxedge A","Cutoff for the tabulated potential interactions in A"}},
	{"TEMPERATURE",{{""},"OPTIONAL",{"NBPOT"},{"real value"},std::to_string(TEMPERATURE_DEF),"Temperature in K, no algorithmic meaning, for tabulated potential"}},
	{"IND_SIGMA_FILE",{{"IND_SIGMA-MASTER_FILE","IND_SIGMA-SCALABLE_FILE"},"One of them is mandatory",{"NBPOT"},{"integer, real, string"},"","The index of the partial for this tabulated potential, sigma, file name with tabulated potential"}},
	{"IND_SIGMA-MASTER_FILE",{{"IND_SIGMA_FILE","IND_SIGMA-SCALABLE_FILE"},"One of them is mandatory",{"NBPOT"},{"integer, real, string"},"","The index of the partial for this tabulated potential, sigma, file name with tabulated potential"}},
	{"IND_SIGMA-SCALABLE_FILE",{{"IND_SIGMA_FILE","IND_SIGMA-MASTER_FILE"},"One of them is mandatory",{"NBPOT"},{"integer, real, string"},"","The index of the partial for this tabulated potential, sigma, file name with tabulated potential"}},
	{"COMP-OPTION",{{""},"OPTIONAL",{"BPOT"},{"strings"},"NONE","A compiler option to use during the reading of the topology file, there can be more, than one"}},
	{"LEAD-BOND",{{"LEAD-ANGLE","LEAD-PERDIH","LEAD-HARMDIH","LEAD-RBDIH"},"One of them is mandatory",{"BPOT"},{"one integer number"},"Setting it to " + std::to_string(LEAD_SERIES_IND2_DEF) + " counting all the potential-related series","The index of the leading potential interaction in its own type"}},
	{"LEAD-ANGLE",{{"LEAD-BOND","LEAD-PERDIH","LEAD-HARMDIH","LEAD-RBDIH"},"One of them is mandatory",{"BPOT"},{"one integer number"},"Setting it to " + std::to_string(LEAD_SERIES_IND2_DEF) + " counting all the potential-related series","The index of the leading potential interaction in its own type"}},
	{"LEAD-PERDIH",{{"LEAD-BOND","LEAD-ANGLE","LEAD-HARMDIH","LEAD-RBDIH"},"One of them is mandatory",{"BPOT"},{"one integer number"},"Setting it to " + std::to_string(LEAD_SERIES_IND2_DEF) + " counting all the potential-related series","The index of the leading potential interaction in its own type"}},
	{"LEAD-HARMDIH",{{"LEAD-BOND","LEAD-ANGLE","LEAD-PERDIH","LEAD-RBDIH"},"One of them is mandatory",{"BPOT"},{"one integer number"},"Setting it to " + std::to_string(LEAD_SERIES_IND2_DEF) + " counting all the potential-related series","The index of the leading potential interaction in its own type"}},
	{"LEAD-RBDIH",{{"LEAD-BOND","LEAD-ANGLE","LEAD-PERDIH","LEAD-HARMDIH"},"One of them is mandatory",{"BPOT"},{"one integer number"},"Setting it to " + std::to_string(LEAD_SERIES_IND2_DEF) + " counting all the potential-related series","The index of the leading potential interaction in its own type"}},
	{"IND_ANN-FILE_TYPE-NAME",{{""},"MANDATORY",{"AENET"},{"integer, string, string"},"","Atom type index, ANN potential file name, optionally atom type name, only required, if it is different than in CHEMICAL-SYMBOLS"}},
	{"WRITE-ATOMIC-ENERGY",{{""},"OPTIONAL",{"AENET"},{"0","1"},std::to_string(WRITE_ATOMIC_ENERGY_DEF),"Whether to write energy for all the atoms in the *.en file"}},
	{"CUTOFF-AENET",{{""},"OPTIONAL",{"AENET"},{"real number"},"same as used in potential construction","Cutoff for the ANN potential in A"}},
	{"AENET-STEP",{{""},"OPTIONAL",{"AENET"},{"integer number"},std::to_string(AENET_STEP_DEF),"Calculate ANN potential in each AENET-SETP simulation step"}}
	});

